# FIND PVALUES USING PERMUTATION TEST BASED ON CORRELATION.  The first two
# arguments are the matrix of feature values, defaulting to "train.x", and
# the vector of class labels, defaulting to "train.y".  If the "ranks" argument
# is TRUE, the ranks of features values are used rather than the original 
# values.  The "perms" argument gives the number of permutations to use.
#
# NOTE:  Because exact ties for correlations may sometimes occur, it is 
# possible that a p-value returned will exceed one.  This is innoccuous.

perm.pval <- function (x=train.x, y=train.y, ranks=FALSE, perms=4999)
{
  if (ranks)
  { x <- apply(x,2,rank)
  }

  actual <- as.vector(cor(y,x))

  p.values.low <- rep(1,ncol(x))
  p.values.high <- rep(1,ncol(x))

  for (i in 1:perms)
  { per <- as.vector(cor(sample(y),x))
    p.values.low <- p.values.low + (per<=actual)
    p.values.high <- p.values.high + (per>=actual)
  }

  2 * pmin (p.values.low, p.values.high) / (perms+1)
}


# FIND PVALUES FOR BINARY FEATURES USING HYPERGEOMETRIC PROBABILITIES.  The 
# arguments are the number of 1 values for each feature in cases where the
# class is 0 and the number of 1 values for each feature in cases where the
# class is 1, the total number of cases, and the number of cases in which
# the class is 1.  These arguments default to the values computed from train.x 
# and train.y.  
#
# NOTE:  As for perm.pval, it is possible that a p-value returned will 
# exceed one.  This is innoccuous.

hyper.pval <- function (n0=apply(train.x[train.y==0,],2,sum),
                        n1=apply(train.x[train.y==1,],2,sum),
                        N=length(train.y), K=sum(train.y))
{
  n <- n0+n1

  p.low <- phyper(n1,K,N-K,n)
  p.high <- phyper(n0,N-K,K,n)
 
  2 * pmin(p.low,p.high) 
}


# FIND PVALUES USING RUNS TESTS.  The first two arguments are the matrix of 
# feature values, defaulting to "train.x", and the vector of class labels,
# defaulting to train.y.  The "nbrk" argument is the number of random 
# tie-breaks to average over.  The "perms" argument is the number of random
# permutations to use for each random tie break.  
#
# WARNING:  The tie breaking scheme depends on the feature values being 
# integers.

runs.pval <- function (x=train.x, y=train.y, nbrk=10, perms=499)
{
  n <- nrow(x)
  m <- ncol(x)

  o <- matrix(0,nbrk,n)
  p.values <- rep(0,m)  

  # Loop over features.

  for (k in 1:m)
  {
    # Find "nbrk" reorderings of this feature with ties broken randomly.

    for (i in 1:nbrk)
    { o[i,] <- order(x[,k]+runif(n))
    }

    # Average the number of 0/1 switches for actual class values as ordered 
    # by each of the orderings of the feature with random breakings of ties.

    da <- 0

    for (i in 1:nbrk)
    { yo <- y[o[i,]]
      da <- da + sum(yo[1:(n-1)]!=yo[2:n])
    }

    da <- da/nbrk

    # Look at average numbers of 0/1 switches for class values randomly 
    # permuted, and add to the p-value when this is less than or equal to 
    # the average for actual counts computed above.

    p.values[k] <- 1   # Start at 1 to account for the actual value

    for (j in 1:perms)
    { yp <- sample(y)
      dp <- 0
      for (i in 1:nbrk)
      { yo <- yp[o[i,]]
        dp <- dp + sum(yo[1:(n-1)]!=yo[2:n])
      }
      dp <- dp/nbrk
      p.values[k] <- p.values[k] + as.numeric (dp<=da)
    }

    # Divide by total possible count to get final p-value.

    p.values[k] <- p.values[k] / (perms+1)

  }

  p.values
}
